
/*	FL03.C 	*/

/*

	FLIST Utility  
	
	Copyright (C) JXL Systems Inc. 1987.  All rights reserved.

	Source file 3 of 3
	
*/

#include "syshdrs.h"    /* standard compiler headers */
#include "textlib.h"	/* the TextLib header files */

#include "flist.h"		/* the FLIST header file */


/***********************************************************************/
/*                                                                     */
/*	This source-file contains functions for:                           */
/*       . file edit                                                   */
/*       . file browse                                                 */
/*       . file copy                                                   */
/*       . file delete                                                 */
/*       . file rename                                                 */
/*		 . create new file using the FLIST text-editor                 */
/*       . confirmations of "dangerous" actions                        */
/*       . sorting the directory-listing                               */
/*                                                                     */
/***********************************************************************/


/***********************************************************************/
/*                                                                     */
/*	These are the BROWSE functions                                     */
/*                                                                     */
/***********************************************************************/

/*	Function BR: browse a file */

br()

{

char brfile[50];
char brfiled[80];

	
	/*
	Clear fields at top right of screen 
	*/

	clrstat();
	
	/*
	Write instructions at bottom of screen
	*/

	f1hlp();
		
	/*
	Get filename
	*/
	
	strcpy(brfile,pathpart);        /* get path part of name */
	strcat(brfile,selfile -> name); /* append selected filename */
	if (selfile -> ext[0]) 
	{
		strcat(brfile,".");
		strcat(brfile,selfile -> ext);
	}

	strcpy(brfiled," Browse: File ");
	strcat(brfiled,brfile);
	strcat(brfiled,"\xff\xc0");
	Display(FunctionDisplayWindow,brfiled);	/* display action being taken */
	
	/*
	Browse the file...
	
	The TextLib FfBrowseFile() function handles everything and
	remains in control until the user presses and end-type key.
	*/
	
	while (1) 
	{
	
		kpfunret = FfBrowseFile(brfile,kcode);
		
		if (! kpfunret) 
		{
			toobig();		/*	error -- file may be too big */
			FfBrowseQuit();
			break;
		}

		if ((kcode == ESC) || (kcode == A_Q))
		{
			FfBrowseQuit();
			break;
		}
		
		if (kcode == U_F1)  	/* help-request */
			edbrhelp(brhelp);	/* display help panel */
		
	}
	
	UnstackAction();    	/* unstack this action */
	
	return(1);
	
}

/*

  	This function is called by the TextLib browse routine
  	each time an input key is struck.  (If it had been
  	omitted, TextLib executes a dummy replacement).  It
  	displays the cursor row and column.
  	
*/  
  
int kffbu1(master,ia,ff,kcode)

Master *master;
Action *ia;
Freeform *ff;
int kcode;

{

int  row;
int  column;   
char rowmsg[80];

	row = FfTextRow(ff) + 1;
	column = FfTextColumn(ff) + 1;

	sprintf(rowmsg," Row %d Col %d\xff\xc0",row,column);
	
	Display(StatusWindow_2,rowmsg);

	return(1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are the EDIT functions                                       */
/*                                                                     */
/***********************************************************************/

/*	Function ED: edit a file */

ed()

{

char edfile[50];
char edfiled[80];
char bakfile[50];
int  cont_edit;

	
	/*
	Clear fields at top right of screen 
	*/

	clrstat();
	
	/*
	Write instructions at bottom of screen
	*/

	f1hlp();
		
	/*
	Get filename
	*/
	
	strcpy(edfile,pathpart);	/* get path part of name */
	strcat(edfile,edfile_name); /* append selected filename */
	strcpy(bakfile,edfile);
	if (edfile_ext[0]) 
	{
		strcat(edfile,".");
		strcat(edfile,edfile_ext);
	}
	strcat(bakfile,".BAK");

	strcpy(edfiled," Edit: File ");
	strcat(edfiled,edfile);
	strcat(edfiled,"\xff\xc0");
	Display(FunctionDisplayWindow,edfiled);	/* display action being taken */
	
	/*
	Edit the file...
	
	The TextLib FfEditFile() function handles everything and
	remains in control until the user presses and end-type key.
	*/

	cont_edit = 1;
	
	while (cont_edit) 
	{
	
		kpfunret = FfEditFile(edfile,kcode);
		
		if (! kpfunret) 
		{
			toobig();		/*	Error -- file may be too big */
			FfEditAbandonChanges();
			break;
		}
			
		switch (kcode) 		/*	depending on ending-key */
		{
		
		case	A_K:	/* save and done */
			FfEditFileSaveAndDone(edfile,bakfile);
			cont_edit = 0;
			break;
			
		case	A_Q:	/* abandoned */
		case	ESC:
			if ((! FfChangesMade(kedit)) || edabort()) 
			{
				FfEditAbandonChanges();
				cont_edit = 0;
			}
			break;

		case	U_F1:	/* Help request */
			edbrhelp(edhelp);
			break;
		
		}
			
	}
	
	UnstackAction();    	/* unstack this action */
	
	return(1);
	
}

/*

  	This function is called by the TextLib edit routine
  	each time an input key is struck.  (If it had been
  	omitted, TextLib executes a dummy replacement).  It
  	displays the row and column of the cursor and whether
  	insert-mode is set.
  	
*/  
  
int kffeu1(master,ia,ff,kcode)

Master *master;
Action *ia;
Freeform *ff;
int kcode;

{

int  row;
int  column;
char rowmsg[80];

	row = FfTextRow(ff) + 1;
	column = FfTextColumn(ff) + 1;

	sprintf(rowmsg," Row %d Col %d %s\xff\xc0",row,column,
		FfAutoIndentMode(ff) ? "Indent" : " ");
	
	Display(StatusWindow_2,rowmsg);
	
	if (FfInsertMode(ff))
		Display(StatusWindow_1," INSERT\xff\xc0");
	else
		Clear(StatusWindow_1);

	return(1);
	
}

edabort()                    /* confirmation of edit-abort */

{

	conflag = confirm("Changes made.  Abandon anyway?");
	return(conflag);           /* return to TextLib function */
	
}


/***********************************************************************/
/*                                                                     */
/*	These are FILE-LIST-SORTING functions                              */
/*                                                                     */
/***********************************************************************/


smentry()	/* entry to menu asking for type of sort desired */

{

	menuinst();		/* display instructions */
	UpPopup(SortMenuPopup);
	MenuMount(); /* display the what-type-of-sort menu */
	return(1);
	
}


sm()	/* service for menu requesting type of sort */

{

int (*compfun)();

	while (1)   /* until user makes a selection */
	{
	
		kpfunret = MenuInput(ipsort);
		
		if (kpfunret >= -1)
			break;	/* values < -1 are PF keys; not used in this menu */
	
	}
	
	if (kpfunret == -1) 	/* ESC */
	{
		UnstackAction();    	/* TextLib -- unstack this action */
		return(1);			/* return to main loop */
	}
	
	/*	kpfunret is positive */

	/*	Use menu choice to select a qsort comparison function */

	compfun = compfuncs[MenuSelection(sortmenu) - 1];
	qsort((char *)filevec,numfiles,sizeof(struct filedesc *),compfun);
	sortflag = 1;	/* indicate directory-listing has been sorted */
	UnstackAction();
	return(1);
	
}

smexit()	/* exit from sort-type menu */

{

	DownPopup(SortMenuPopup); /* take down the popup used by the menu */
	return(1);
	
}

/*

	Comparison functions for the sort
	
*/

int compname(a1,a2)

struct filedesc **a1;
struct filedesc **a2;

{

static int (*cfname[4])() = { cname, cext, cdate, csize };

	/* cfname[0] -> [3] = major -> minor sort fields */
	
	return(cprocess(cfname,a1,a2));
	
}

int compext(a1,a2)

struct filedesc **a1;
struct filedesc **a2;

{

static int (*cfext[4])() = { cext, cname, cdate, csize };

	/* cfext[0] -> [3] = major -> minor sort fields */

	return(cprocess(cfext,a1,a2));
	
}

int compdate(a1,a2)

struct filedesc **a1;
struct filedesc **a2;

{

static int (*cfdate[4])() = { cdate, cname, cext, csize };

	/* cfdate[0] -> [3] = major -> minor sort fields */

	return(cprocess(cfdate,a1,a2));
	
}

int compsize(a1,a2)

struct filedesc **a1;
struct filedesc **a2;

{

static int (*cfsize[4])() = { csize, cname, cext, cdate };

	/* cfsize[0] -> [3] = major -> minor sort fields */

	return(cprocess(cfsize,a1,a2));
	
}

int cprocess(funclist,a1,a2)

int (*funclist[])();
struct filedesc **a1;
struct filedesc **a2;

{

int i;
int result;

	for (i = 0; i < 4; ++i) 
	{
		result = (*funclist[i])(a1,a2);	/* do next comparison */
		if (result)	/* if unequal result */
			break;
	}
	
	return(result);
	
}

cname(a1,a2)	/* compare filenames */

struct filedesc **a1;
struct filedesc **a2;

{

	return(strcmp((*a1) -> name,(*a2) -> name));
	
}

cext(a1,a2)	/* compare extensions */

struct filedesc **a1;
struct filedesc **a2;

{

	return(strcmp((*a1) -> ext,(*a2) -> ext));
	
}

cdate(a1,a2)	/* compare dates (most recent first) */

struct filedesc **a1;
struct filedesc **a2;

{

int result;

	result = (*a2) -> date - (*a1) -> date;
	if (result)
		return(result);
	return((*a2) -> time - (*a1) -> time);
	
}

csize(a1,a2)	/* compare filesizes (largest first) */

struct filedesc **a1;
struct filedesc **a2;

{

long difference;

	difference = ((*a2) -> size).sizelong - ((*a1) -> size).sizelong;
	
	if (! difference)
		return(0);
		
	return(difference > 0 ? 1 : -1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are FILE-DELETE functions                                    */
/*                                                                     */
/***********************************************************************/


delfile()

{
	
int renstat;
	
	if (conflag)    /* if delete was confirmed */
		renstat = remove(file2);
		
	conflag = 0;

	if (renstat == -1) 
	{
		Display(StatusWindow_1," Delete failed\xff\xc0");
		Display(StatusWindow_3," Press any key to continue...\xff\xc0");
		if (! getch())
			getch();
		clrstat();
	}

	return(1);
	
}


/***********************************************************************/
/*                                                                     */
/*	These are COPY functions                                           */
/*                                                                     */
/***********************************************************************/


cpentry()	/* entry to: to-file for copy */

{

char fname[50];

	UpPopup(CopyToFilenamePopup);
	Display(CopyToFilename_1,
	  "Copy to (full path name):\xff\xc0");
	return(1);
	
}

cpres()		/* action resumed after the "confirm" action */

{

	UnstackAction();	/* now unstack this action to cause cpexit() to go */
	return(1);
	
}


cf()	/* get the to-file name */

{

int tolen;
int cont_input;
char *tofile;
char fname[80];

	/* 
	This function gets the to-file name and
	requests confirmation if the to-file already exists.  The
	actual copying is done in the cpexit() function if conflag is
	set; i.e., if the copy is to proceed.
	*/

	cont_input = 1;

	conflag = 0;	/* set not-ok-to-proceed for copy */

	while (cont_input) 
	{

		StInput(kcode);               /* get to-file name */
	
		StInputOrEditDone(tofile,tolen); /* acquire data, length */
	
		switch (kcode)  /* User's terminating keypress */
		{
		
		case	ESC:	/* user wants to forget it */
			
			cont_input = 0;
			UnstackAction(); /* conflag = 0, so there will be no copy */
			break;
			
		case	ENTER:      /* user has entered a filename */

			if (! tolen)  /* to-file name cannot be null */
				break;

			strcpy(fullfn2,tofile);
			fnsplit(fullfn2,drive2,dir2,fnm2,ext2);
			if (! (drive2[0]))
				strcpy(drive2,drive);
			if (! (dir2[0]))
				strcpy(dir2,dir);
			fnmerge(file2,drive2,dir2,fnm2,ext2);
			
			cont_input = 0;	

			/* check if the to-file currently exists */
				
			if (! access(file2,0))  /* if file exists */
			{
				strcpy(fname,"File ");
				strcat(fname,file2);
				strcat(fname, " already exists. Proceed anyway?");
				conflag = confirm(fname);
					
			}
				
			else 					/* file doesn't exist */
				conflag = 1;  /* set ok-to-proceed with copy */

			UnstackAction();

			break;
				
		}
		
	}

	return(1);
	
}

cpexit()	/* copy-file exit (does the actual copy if ok to go) */

{

char fromfile[50];
int  copyerr;
int  readcount;
int  writecount;
char *xfrbuf;
int  eofflag;
int  maxxfr;
int  fromopen;
int  toopen;
static char *copy_errors[] = 
{
	"Could not open from-file",
	"Could not create to-file",
	"Not enough storage for copy-buffer",
	"Error writing to-file"
};

	copyerr = 0;
	xfrbuf = 0;
	fromopen = 0;
	toopen = 0;
	
	if (conflag)   /* if copy is to take place */
	{
	
		conflag = 0;	/* reset this flag */
	
		strcpy(fromfile,pathpart);	/* path portion of from file */
		strcat(fromfile,selfile -> name);
		if (selfile -> ext[0]) 
		{
			strcat(fromfile,".");
			strcat(fromfile,selfile -> ext);
		}
		
		if (! (fd1 = fopen(fromfile,"rb")))
			copyerr = 1; 
		else
			fromopen = 1;

		if (! copyerr) 
		{
			if (! (fd2 = fopen(file2,"wb")))
				copyerr = 2; 
			else
				toopen = 1;
		}			

		if (! copyerr) 
		{
			maxxfr = 4096;	/* transfer in 4K chunks */
			if (! (xfrbuf = malloc(maxxfr))) /* get a buffer */
				copyerr = 3;
		}

		eofflag = copyerr; 
		
		while (! eofflag) 
		{
		
			readcount = fread(xfrbuf,1,maxxfr,fd1);
			if (readcount != maxxfr)
				eofflag = 1;
				
			if (readcount) 
			{
				writecount = fwrite(xfrbuf,1,readcount,fd2);
				if (writecount != readcount) 
				{
					copyerr = 4;
					eofflag = 1;
				}
			}
			
		}
		
		if (xfrbuf)
			free(xfrbuf);
			
		if (fromopen)
			fclose(fd1);
		if (toopen)
		{
			if (fclose(fd2))
				copyerr = 4;
		}	
			
	}	
	
	DownPopup(CopyToFilenamePopup);

	if (copyerr) 
	{
		FastMsg("a",copy_errors[copyerr - 1]);
	}

	return(1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are RENAME functions                                         */
/*                                                                     */
/***********************************************************************/

rnentry()	/* entry to: to-file for rename */

{

	UpPopup(RenameToFilenamePopup);
	Display(RenameToFilename_1,"New name (full path):\xff\xc0");
	return(1);
	
}

rf()	/* get new-name for rename */

{

int tolen;
int cont_input;
char *tofile;
char fromfile[50];

	cont_input = 1;

	while (cont_input) 
	{

		StInput(kcode);               /* get new name */
	
		StInputOrEditDone(tofile,tolen);
	
		switch (kcode)  /* User's terminating keypress */
		{
		
		case	ESC:	/* user wants to forget it */
			
			cont_input = 0;
			UnstackAction();
			break;
			
		case	ENTER:	/* user has entered a filename */

			if (! tolen) /* cannot rename to a null name */
				break;

			strcpy(fullfn2,tofile);
			fnsplit(fullfn2,drive2,dir2,fnm2,ext2);
			if (! (drive2[0]))
				strcpy(drive2,drive);
			if (! (dir2[0]))
				strcpy(dir2,dir);
			fnmerge(file2,drive2,dir2,fnm2,ext2);
			
			cont_input = 0;	
		
			strcpy(fromfile,pathpart);	/* get pathname of from-file */
			strcat(fromfile,selfile -> name);
			if (selfile -> ext[0]) 
			{
				strcat(fromfile,".");
				strcat(fromfile,selfile -> ext);
			}
				
			if (rename(fromfile,file2))		/* rename the file */
				FastMsg("a","Rename failed");
				
			UnstackAction();
					
			break;
				
		}
		
	}

	return(1);
	
}

rnexit()	/* rename-file exit */

{
	
	DownPopup(RenameToFilenamePopup);
	return(1);
	
}

/***********************************************************************/
/*                                                                     */
/*	These are CREATE-NEW-FILE file-name-specification functions        */
/*                                                                     */
/***********************************************************************/

cnentry()	/* entry to: filename for create */

{

	UpPopup(CreateNewFilePopup);
	Display(CreateNewFile_1,
		"Name of file to create (in this directory):\xff\xc0");
	return(1);
	
}

cn()	/* get filename to create */

{

int tolen;
int cont_input;
char *tofile;
char *dot_pos;
int  name_len;
int  ext_len;
char *ext_pos;

	/* Note: this function is called from another function rather than main */
	/* Thus, it does not issue an UnstackAction() under any circumstances */

	cont_input = 1;
	
	cn_ok = 0;		/* set no-go switch for create */

	while (cont_input) 
	{

		StInput(kcode);               /* get filename */
	
		StInputOrEditDone(tofile,tolen);
	
		switch (kcode)  /* User's terminating keypress */
		{
		
		case	ESC:	/* user wants to forget it */
			
			cont_input = 0;
			break;
			
		case	ENTER:	/* user has entered a filename */


			if (! tolen) /* cannot create a null name */
			{
				FastMsg("a","Null File Name");
				break;
			}

			if (tolen > 12)	/* illegal filename */
			{
				FastMsg("a","Filename too long");
				break;
			}

			if (strchr(tofile,' ') || strchr(tofile,'\t') ||
			    strchr(tofile,':') || strchr(tofile,'\\'))
			{
			    FastMsg("a","Drive, path not allowed");
				break;
			}
				
			name_len = ((dot_pos = strchr(tofile,'.')) ?
			  dot_pos - tofile : tolen);
				  
			 if ((! name_len) || (name_len > 8))
			 {
			 	FastMsg("a","Illegal Filename");
			 	break;
			 }

			 fnsplit(tofile,drive2,dir2,fnm2,ext2);
			 fnmerge(fullfn2,drive,dir,fnm2,ext2);
			 if (! access(fullfn2,0))	/* if file already exists */
			 {
			 	FastMsg("a","File already exists");
			 	break;
			 }
				 	
			 memcpy(edfile_name,tofile,name_len);
			 edfile_name[name_len] = 0;
				 	
			 if (dot_pos) 
			 {
			 	ext_pos = dot_pos + 1;
			 	ext_len = strlen(ext_pos);
			 	if ((! ext_len) || (ext_len > 3))
			 	{
			 		FastMsg("a","Illegal extension");
			 		break;
			 	}
			 	strcpy(edfile_ext,ext_pos);
			 }
			 else
			 	edfile_ext[0] = 0;
					
			cont_input = 0;	
				
			cn_ok = 1;	/* set ok-to-go */
		
			break;
				
		}

	}

	return(1);
	
}

cnexit()	/* create-file exit */

{
	
	DownPopup(CreateNewFilePopup);
	return(1);
	
}





